//WARNING comment syntax is important,  if unsure, check out http://www.doxygen.org/manual.html

//---------------------------------------------------------------------------
//! \file drivers.h
//!
//! \brief  Publicly assessable functions and types for drivers.c
//!
//! Garage project: device drivers
//!
//! \author David R Brooks
//! \date 18/06/2007
//!
//! \attention           CPU: Atmel ATmega08
//!
//---------------------------------------------------------------------------
// 2007.09.24 18:44  [YYYY.MM.DD] Last edited (CodeWright)

#ifndef __DRIVERS_H
#define __DRIVERS_H

//-------------------------------------------------------------------------
// Symbols
//-------------------------------------------------------------------------
#define SETBIT(port, bit, val)	{if(val) port |= BV(bit); else port &= ~BV(bit);}

#define CLKSCALE   8        // Clock prescale: Fclk = 8kHz

//-------------------------------------------------------------------------
// Input lines
//-------------------------------------------------------------------------
#define BUTTON 		  (!(PIND & BV(PD3)))		// Pushbutton: active low
#define RX_A			(PIND & BV(PD6))		// Radio channels: small fob button
#define RX_B			(PINB & BV(PB0))		//				   large fob button
#define RX_C			(PINB & BV(PB1))
#define RX_D			(PINB & BV(PB2))
#define RX_ALL 			(PIND & BV(PD5))		// OR of channels A-D
#define INDEX			(PINC & BV(PC2))		// 6-toothed wheel on clutch shaft

//-------------------------------------------------------------------------
// External switch inputs. Sensed via the ADC, to detect broken wires, etc.
//-------------------------------------------------------------------------
// Values good for rail voltage 16 - 18.5V
#define ADCRAW(x)   (uint2)((x*1023L)/CFG_ADCREF_MV)   // ADC reading for given voltage (mV)

// ADC input channels (external, failsafe, switches)
#define NADCS           2                       // No. of ADC channels used
#define LIMTOP_ADC		6						// Top limit switch
#define LIMBTM_ADC		7						// Bottom limit switch

#define START_ADC(x) {ADMUX = (x); ADCSRA |= BV(ADSC);}

//-------------------------------------------------------------------------
// Output lines (driver macros)
//-------------------------------------------------------------------------
#define SETRLYDOWN(x)	SETBIT(PORTC, PC1, x)	// Drive curtain down
#define SETLEDBTN(x)	SETBIT(PORTC, PC3, x)	// Pushbutton LED
#define SETLEDRED(x) 	SETBIT(PORTC, PC4, x)	// Red LED
#define SETLEDGRN(x)	SETBIT(PORTC, PC5, x)	// Green LED
#define SETRLYSTART(x)	SETBIT(PORTB, PB6, x)	// Motor-start relay
#define SETRLYPOWER(x)	SETBIT(PORTB, PB7, x)	// Motor-power relay
#define SETWATCHDOG(x)	SETBIT(PORTD, PD4, x)	// External watchdog tickler
#define SETLEARN(x) 	SETBIT(DDRD,  PD7, x)	// Put RX into LEARN mode

#define INIT_DDR_B (BV(PB6) | BV(PB7))
#define INIT_DDR_C (BV(PC1) | BV(PC3) | BV(PC4) | BV(PC5))
#define INIT_DDR_D (BV(PD4))

// Set outputs, & record them. Be sure to set flag first (so we can toggle it)
#define OUT_DOWN(x)  {SETRLYDOWN(x); }  // DON'T set i_down!
#define OUT_START(x) {i_start = (x); SETRLYSTART(x);}
#define OUT_POWER(x) {i_power = (x); SETRLYPOWER(x);}
#define OUT_LEARN(x) {i_learn = (x); SETLEARN(x);}

//-Global Externs ------------------------------------------------------------

//-Global Function Prototypes ------------------------------------------------
void   StartHardware(void);
void   IoPortInit(void);
void   AdcInit(void);
void   TimebaseInit(uint2 period_u2);
bool1  IsRadio(void);

#endif //__DRIVERS_H

