//WARNING comment syntax is important,  if unsure, check out http://www.doxygen.org/manual.html

//---------------------------------------------------------------------------
//! \file main.h
//!
//! \brief  Publicly accessible functions and types for main.c
//!
//! General definitions
//!     Version:    1.04         
//!     Processor:  Atmel ATMega8
//!     Compiler:   GCC          
//!
//! \author David R Brooks
//! \date 14/07/2007
//!
//---------------------------------------------------------------------------
// 2007.10.06 14:41  [YYYY.MM.DD] Last edited (CodeWright)

/*  CAUTION: We are quite tight on RAM usage, due to large confign.
    tables in main.c. If the code is to be further extended, these
    should be moved into Flash, & accessed via the PROGMEM macros.

    Security/Configuration Bits:
    Tick the following bits in PonyProg's window:
        [SPIEN]
        BOOTSZ1
        BOOTSZ0
        SUT0
        CKSEL3
        CKSEL1
        CKSEL0
*/

#ifndef __MAIN_H
#define __MAIN_H

//-Included files -----------------------------------------------------------
#include "typedefn.h"
#include "interrupt.h"
#include "drivers.h"

//-Symbols -------------------------------------------------------------------
#define PROJECT     "Garage"
#define DESCRIPTION "Door Remote Control"
#define REVMAJOR	"2"
#define REVMINOR    "02"
#define AUTHOR      "David R Brooks"
#define YEAR        "2007"

#define _WDR()      __asm__ __volatile__ ("wdr" ::)
#define ADCSENSE    FALSE	// ADC switch sense (inversion)

//-Configuration constants --------------------------------------------------
#define CFG_SIGNON (PROJECT "-" DESCRIPTION " v" REVMAJOR "." REVMINOR "  Copyright (c) " YEAR ". " AUTHOR)

#define CFG_STARTDEL_US		  1000		// Start-up delay for UART 
#define CFG_UPDATE_MS		    20		// Main state machine update period
#define CFG_COMMRATE_BD		 57600		// Debug port bauds
#define CFG_RXCHAN_E        CHAN_B		// Radio receiver channel in use

#define CFG_MOTIONTIMEOUT_MS 10000		// Max. time allowed for motion
#define CFG_PICKDELAY_MS       100      // Wait for small relays, before applying power
#define CFG_STARTDELAY_MS      250      // Wait for motor to start, then release start wdg.
#define CFG_CLUTCHTIMEOUT_MS  1000      // Wait for shaft to move (allow for disengaged clutch)
#define CFG_CLUTCHRUNTIME_MS   250      // Time between successive index pulses (stall test)

// ADC trip values, simulating switches with diagnostics.
// NB The read values dip sharply as the motor starts (design error: the resistor strings
//    are fed from the unregulated supply, which dips as the motor starts). A "high"
//    reading can be as low as 2900mV.
#define CFG_ADCREF_MV        5000L      // ADC reference
#define CFG_ADCMAX_MV        4300L      // Highest valid "1" reading
#define CFG_ADCHIGH_MV       2900L      // Lowest ditto
#define CFG_ADCLOW_MV        2700L      // Highest valid "0" reading
#define CFG_ADCMIN_MV        0700L      // Lowest ditto

#define CFG_LED_IDLE_ON_MS		50		// LED blink times
#define CFG_LED_IDLE_OFF_MS	  1000
#define CFG_LED_FAULT_ON_MS		75
#define CFG_LED_FAULT_OFF_MS	75

//---------------------------------------------------------------------------

typedef enum 			// Main state machine
{
	s_idle  = 0,		// Ready
	s_pick,				// Set the start/dirn. relays
	s_start,			// Motor ON, start active
	s_index,			// Waiting for 1st Index
	s_run,				// Running...
	s_end,			    // Stop moving
	s_fault,			// Fatal errors	(3 states)
    s_catch,
    s_exit
} E_CSTATE;	

typedef enum        // Radio channel selector
{
    CHAN_A,             // Small fob button
    CHAN_B,             // Large fob button
    CHAN_C,
    CHAN_D,
    CHAN_ALL
} E_RADIO;

typedef enum        // Swtch inputs (which need debouncing)	must match adc_chan_u1
{
    S_LIM_TOP,          // Top limit switch
    S_LIM_BTM,          // Bottom limit switch
    S_INDEX,            // Shaft rotation sensor 
    S_RADIO,            // Radio input
    S_BUTTON,           // Pushbutton input
    S_ERROR_TOP,        // Pseudo-inputs: ADC range errors
    S_ERROR_BTM,
    S_SWITCHES          // Must be last
} E_INPUT;
						// Order should match the above
#define SIGNAMES {"Top", "Bottom", "Index", "Radio", "Button", "Err-Top", "Err-Btm", "" }

#define SENSOR(x) (InputsFiltered_u1 & BV(x))
#define EDGE(x)   (InputsEdge_u1     & BV(x))

//-Global Externs ------------------------------------------------------------

//-Global Function Prototypes ------------------------------------------------

void  StateMachine(void);
void  ExternalInputs(void);

#endif /* __MAIN_H */

