/********************************************************************
*                                                                   *
*           G A R A G E  -  D O O R   C O N T R O L L E R           *
*                                                                   *
*                   C O N T R O L   M O D U L E                     *
*                                                                   *
*********************************************************************
*                                                                   *
* File:       TYPEDEFN.H                                            *
* Purpose:    Basic data types                                      *
* Author:     David R Brooks                                        *
* Date:       May, 2005                                             *
* Version:    1.01                                                  *
* Processor:  Atmel ATMega8                                         *
* Compiler:   GCC                                                   *
*                                                                   *
*********************************************************************/
// 2007.09.23 14:51  Last edited (CodeWright)

#ifndef __TYPEDEFN_H
#define __TYPEDEFN_H

/*----- INCLUDES ---------------------------------------------------------*/

#include <avr/io.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>

/*----- DEFINES ----------------------------------------------------------*/

/*
typedef enum
{
    UINT1 = 0,
    SINT1 = 1,
    UINT2 = 2,
    SINT2 = 3,
    UINT4 = 4,
    SINT4 = 5,
    REAL4 = 6,
    REAL8 = 7 
} E_TypeID;
*/

#ifndef BV
    #define BV(X) (1 << (X))
#endif

#ifndef NULL
    #define NULL ((void*)0)
#endif


#define HIGH(VAR) (VAR >> 8)
#define LOW(VAR)  (VAR & 0xff)

#define flash       __attribute__((__progmem__))
#define eeprom      __attribute__((section(".eeprom")))
#define calibration __attribute__((section(".calibration")))
#define bootloader  __attribute__((section(".bootloader")))
#define bootentry   __attribute__((section(".bootentry")))
#define extmem      __attribute__((section(".extmem")))
#define strtab      __attribute__((section(".strtab")))
#define strdat      __attribute__((section(".strdat")))

#define TRUE  (1)
#define FALSE (0)

// Maximum values for one byte integer types
// 'Minimum' is the minimum value that can be
// used in an ABS (Absolute value) function
#define MAX_UINT1   (0xff)
#define MIN_SINT1   ((sint1)0x81)
#define MAX_SINT1   ((sint1)0x7f)

// Maximum values for two byte integer types
#define MAX_UINT2   (0xffff)
#define MIN_SINT2   ((sint2)0x8001)
#define MAX_SINT2   ((sint2)0x7fff)

// Maximum values for three (type is four) byte integer types
#define MAX_UINT3   (0x00ffffff)
#define MIN_SINT3   ((sint4)0xff800001)
#define MAX_SINT3   ((sint4)0x007fffff)

// Maximum values for four byte integer types
#define MAX_UINT4   (0xffffffff)
#define MIN_SINT4   ((sint4)0x80000001)
#define MAX_SINT4   ((sint4)0x7fffffff)

#define BINARY_MEGA ((uint4)0x00100000) //  1,048,576
#define BIN_POW_24  ((uint4)0x01000000) // 16,777,216
#define BIN_POW_10  ((uint2)0x00000400) //       1024

/*----- TYPEDEFS ---------------------------------------------------------*/

typedef unsigned char bool1;
typedef unsigned char uint1;
typedef signed char   sint1;
typedef unsigned short int  uint2;
typedef signed  short int   sint2;
typedef unsigned long  uint4;
typedef signed   long  sint4;
typedef unsigned long long uint8;
typedef long long sint8;
typedef float real4;
typedef double real8;

// NB AVR is little endian

typedef union
{
    uint4 var;
    uint1 byte[4];
}   U_UQuad;

typedef union
{
    uint2 var;
    uint1 byte[2];
}   U_UDual;

typedef union
{
    sint4 var;
    uint1 byte[4];
}   U_SQuad;

typedef union
{
    sint2 var;
    uint1 byte[2];
}   U_SDual;

/*----- EXTERNALS --------------------------------------------------------*/

#if ((defined INCLUDE_EXTERNS)||(!defined __HEADER_C))

#endif /* (defined INCLUDE_EXTERNS)||(!defined __HEADER_C) */

// Character strings located in external, serial EEROM
// String directory is built at the start.
#define XSTRING(name,text)			 									\
	const uint1 name##_1[] strdat = text;								\
	const uint2 name[] strtab = {(uint2)&name##_1[0], sizeof(name##_1)};

#ifndef DIMOF
#define DIMOF(x) (sizeof(x) / sizeof(x[0]))
#endif

/*----- PROTOTYPES -------------------------------------------------------*/
/*----- INLINE FUNCTIONS -------------------------------------------------*/

#endif /* __TYPEDEFN_H */

/*----- END OF FILE ------------------------------------------------------*/
